#property indicator_chart_window
#property indicator_buffers 4
#property indicator_color1 clrRed
#property indicator_color2 clrBlue
#property indicator_width1 2
#property indicator_width2 2

extern int swing_length = 10;
extern int history_of_demand_to_keep = 20;
extern double box_width = 2.5;

extern bool show_zigzag = false;
extern bool show_price_action_labels = false;

extern color supply_color = clrGray, supply_outline_color = clrWhite;
extern color demand_color = clrCyan, demand_outline_color = clrWhite;
extern color bos_label_color = clrWhite, poi_label_color = clrWhite;
extern color swing_type_color = clrBlack, zigzag_color = clrGray;

double swing_high_values[];
double swing_low_values[];
double supply_array[];
double demand_array[];
double bos_array[];

bool DoubleIsNaN(double value)
{
    return value != value;
}

void 

 OnCalculate(const int rates_total, const int prev_calculated, const datetime &time[], const double &open[], 
    const double &high[], const double &low[], const double &close[], const long &tick_volume[], const long &volume[], 
    const int &spread[])
    


{
    double atr = iATR(NULL, 0, 50, 0);
    double swing_high = iCustom(NULL, 0, "Pivot High Low", swing_length, 0, 0);
    double swing_low = iCustom(NULL, 0, "Pivot High Low", swing_length, 1, 0);

    if (!DoubleIsNaN(swing_high))
    {
         ArrayCopySeries(swing_high_values, 0, 1, 4);
         swing_high_values[0] = swing_high;

        ArraySetAsSeries(supply_array, true);
        int bn = iBarShift(NULL, 0, swing_high);
        double poi = (swing_high + swing_low) / 2;
        bool okay_to_draw =  CheckOverlapping(poi, supply_array, atr);

     // Calculate the new supply zone
if (okay_to_draw)
{
    ArrayCopySeries(supply_array, 1, supply_array, 0, ArraySize(supply_array) - 1);
    ArrayResize(supply_array, ArraySize(supply_array) - 1);
    supply_array[0] = swing_high;
    ObjectCreate("SupplyZone", OBJ_RECTANGLE_LABEL, 0, bn, swing_high, bn + 100, swing_high - atr * box_width);
    ObjectSet("SupplyZone", OBJPROP_COLOR, supply_color);
    ObjectSet("SupplyZone", OBJPROP_STYLE, STYLE_SOLID);
    ObjectSet("SupplyZone", OBJPROP_WIDTH, 2);
}

    }

    // Implement the rest of the calculations similarly as above for the swing_low values and demand zones
    // ...

    if (show_zigzag)
    {
        int shift = 0;
        if (swing_high_values[0] >= swing_high_values[1])
        {
            shift = iBarShift(NULL, 0, swing_high_values[0]);
            ObjectCreate("ZigZag", OBJ_TREND, 0, shift, swing_high_values[0], shift + 1, swing_high_values[1]);
            ObjectSet("ZigZag", OBJPROP_COLOR, zigzag_color);
            ObjectSet("ZigZag", OBJPROP_STYLE, STYLE_SOLID);
            ObjectSet("ZigZag", OBJPROP_WIDTH, 2);
        }
        else if (swing_low_values[0] <= swing_low_values[1])
        {
            shift = iBarShift(NULL, 0, swing_low_values[0]);
            ObjectCreate("ZigZag", OBJ_TREND, 0, shift, swing_low_values[0], shift + 1, swing_low_values[1]);
            ObjectSet("ZigZag", OBJPROP_COLOR, zigzag_color);
            ObjectSet("ZigZag", OBJPROP_STYLE, STYLE_SOLID);
            ObjectSet("ZigZag", OBJPROP_WIDTH, 2);
        }
    }
}

bool f_check_overlapping(double new_poi, double &box_array[], double atr)
{
    double atr_threshold = atr * 2;
    bool okay_to_draw = true;

    for (int i = 0; i < ArraySize(box_array); i++)
    {
        double top = ObjectGet("SupplyZone", OBJPROP_PRICE1, i);
        double bottom = ObjectGet("SupplyZone", OBJPROP_PRICE2, i);
        double poi = (top + bottom) / 2;

        double upper_boundary = poi + atr_threshold;
        double lower_boundary = poi - atr_threshold;

        if (new_poi >= lower_boundary && new_poi <= upper_boundary)
        {
            okay_to_draw = false;
            break;
        }
        else
        {
            okay_to_draw = true;
        }
    }

    return okay_to_draw;
}
